@@ -51,11 +51,8 @@ static inline void bch2_accounting_accumulate_maybe_kill(struct bch_fs *c,
 {
 	bch2_accounting_accumulate(dst, src);
 
-	for (unsigned i = 0; i < bch2_accounting_counters(&dst->k); i++)
-		if (dst->v.d[i])
-			return;
-
-	__bch2_accounting_maybe_kill(c, dst->k.p);
+	if (bch2_accounting_key_is_zero(accounting_i_to_s_c(dst)))
+		__bch2_accounting_maybe_kill(c, dst->k.p);
 }
 
 static inline void fs_usage_data_type_to_base(struct bch_fs_usage_base *fs_usage,
@@ -110,23 +107,20 @@ do {									\
 	(_k)._type	= (struct bch_acct_##_type) { __VA_ARGS__ };	\
 } while (0)
 
-#define bch2_disk_accounting_mod2_nr(_trans, _gc, _v, _nr, ...)		\
+#define bch2_disk_accounting_mod2(_trans, _gc, _v, ...)			\
 ({									\
 	struct disk_accounting_pos pos;					\
 	disk_accounting_key_init(pos, __VA_ARGS__);			\
-	bch2_disk_accounting_mod(trans, &pos, _v, _nr, _gc);		\
+	bch2_disk_accounting_mod(trans, &pos, _v, ARRAY_SIZE(_v), _gc);	\
 })
 
-#define bch2_disk_accounting_mod2(_trans, _gc, _v, ...)			\
-	bch2_disk_accounting_mod2_nr(_trans, _gc, _v, ARRAY_SIZE(_v), __VA_ARGS__)
-
 int bch2_mod_dev_cached_sectors(struct btree_trans *, unsigned, s64, bool);
 
 int bch2_accounting_validate(struct bch_fs *, struct bkey_s_c,
 			     struct bkey_validate_context);
-void bch2_accounting_key_to_text(struct printbuf *, struct disk_accounting_pos *);
+void bch2_accounting_key_to_text(struct printbuf *, struct bch_fs *, struct disk_accounting_pos *);
 void bch2_accounting_to_text(struct printbuf *, struct bch_fs *, struct bkey_s_c);
-void bch2_accounting_swab(struct bkey_s);
+void bch2_accounting_swab(const struct bch_fs *, struct bkey_s);
 
 #define bch2_bkey_ops_accounting ((struct bkey_ops) {	\
 	.key_validate	= bch2_accounting_validate,	\
@@ -152,6 +146,7 @@ enum bch_accounting_mode {
 
 int bch2_accounting_mem_insert(struct bch_fs *, struct bkey_s_c_accounting, enum bch_accounting_mode);
 int bch2_accounting_mem_insert_locked(struct bch_fs *, struct bkey_s_c_accounting, enum bch_accounting_mode);
+void bch2_accounting_mem_gc(struct bch_fs *);
 
 static inline bool bch2_accounting_is_mem(struct disk_accounting_pos *acc)
 {
@@ -236,7 +231,7 @@ static inline int bch2_accounting_mem_mod_locked(struct btree_trans *trans,
 
 static inline int bch2_accounting_mem_add(struct btree_trans *trans, struct bkey_s_c_accounting a, bool gc)
 {
-	guard(percpu_read)(&trans->c->mark_lock);
+	guard(percpu_read)(&trans->c->capacity.mark_lock);
 	return bch2_accounting_mem_mod_locked(trans, a, gc ? BCH_ACCOUNTING_gc : BCH_ACCOUNTING_normal, false);
 }
 
@@ -259,7 +254,7 @@ static inline void bch2_accounting_mem_read_counters(struct bch_accounting_mem *
 static inline void bch2_accounting_mem_read(struct bch_fs *c, struct bpos p,
 					    u64 *v, unsigned nr)
 {
-	guard(percpu_read)(&c->mark_lock);
+	guard(percpu_read)(&c->capacity.mark_lock);
 	struct bch_accounting_mem *acc = &c->accounting;
 	unsigned idx = eytzinger0_find(acc->k.data, acc->k.nr, sizeof(acc->k.data[0]),
 				       accounting_pos_cmp, &p);
