@@ -2,7 +2,37 @@
 #ifndef _BCACHEFS_ERRCODE_H
 #define _BCACHEFS_ERRCODE_H
 
+/* we're getting away from reusing bi_status, this should go away */
+#define BLK_STS_REMOVED		((__force blk_status_t)128)
+
+#define BLK_ERRS()				\
+	BLK_STS(NOTSUPP)			\
+	BLK_STS(TIMEOUT)			\
+	BLK_STS(NOSPC)				\
+	BLK_STS(TRANSPORT)			\
+	BLK_STS(TARGET)				\
+	BLK_STS(RESV_CONFLICT)			\
+	BLK_STS(MEDIUM)				\
+	BLK_STS(PROTECTION)			\
+	BLK_STS(RESOURCE)			\
+	BLK_STS(IOERR)				\
+	BLK_STS(DM_REQUEUE)			\
+	BLK_STS(AGAIN)				\
+	BLK_STS(DEV_RESOURCE)			\
+	BLK_STS(ZONE_OPEN_RESOURCE)		\
+	BLK_STS(ZONE_ACTIVE_RESOURCE)		\
+	BLK_STS(OFFLINE)			\
+	BLK_STS(DURATION_LIMIT)			\
+	BLK_STS(INVAL)				\
+	BLK_STS(REMOVED)			\
+
+#define BLK_STS(n)								\
+	x(BCH_ERR_blockdev_io_error,	BLK_STS_##n)
+
 #define BCH_ERRCODES()								\
+	x(EIO,				blockdev_io_error)			\
+	BLK_ERRS()								\
+	x(BCH_ERR_blockdev_io_error,	BLK_STS_UNKNOWN)			\
 	x(ERANGE,			ERANGE_option_too_small)		\
 	x(ERANGE,			ERANGE_option_too_big)			\
 	x(ERANGE,			projid_too_big)				\
@@ -48,6 +78,7 @@
 	x(ENOMEM,			ENOMEM_bio_read_init)			\
 	x(ENOMEM,			ENOMEM_bio_read_split_init)		\
 	x(ENOMEM,			ENOMEM_bio_write_init)			\
+	x(ENOMEM,			ENOMEM_promote_limit_init)		\
 	x(ENOMEM,			ENOMEM_bio_bounce_pages_init)		\
 	x(ENOMEM,			ENOMEM_writepage_bioset_init)		\
 	x(ENOMEM,			ENOMEM_dio_read_bioset_init)		\
@@ -107,6 +138,7 @@
 	x(ENOSPC,			ENOSPC_sb_replicas)			\
 	x(ENOSPC,			ENOSPC_sb_members)			\
 	x(ENOSPC,			ENOSPC_sb_members_v2)			\
+	x(ENOSPC,			ENOSPC_sb_extent_type_u64s)		\
 	x(ENOSPC,			ENOSPC_sb_crypt)			\
 	x(ENOSPC,			ENOSPC_sb_downgrade)			\
 	x(ENOSPC,			ENOSPC_btree_slot)			\
@@ -153,6 +185,7 @@
 	x(BCH_ERR_transaction_restart,	transaction_restart_upgrade)		\
 	x(BCH_ERR_transaction_restart,	transaction_restart_key_cache_fill)	\
 	x(BCH_ERR_transaction_restart,	transaction_restart_key_cache_raced)	\
+	x(BCH_ERR_transaction_restart,	transaction_restart_lock_root_race)	\
 	x(BCH_ERR_transaction_restart,	transaction_restart_split_race)		\
 	x(BCH_ERR_transaction_restart,	transaction_restart_write_buffer_flush)	\
 	x(BCH_ERR_transaction_restart,	transaction_restart_nested)		\
@@ -174,7 +207,7 @@
 	x(BCH_ERR_btree_insert_fail,	btree_insert_need_journal_reclaim)	\
 	x(0,				backpointer_to_overwritten_btree_node)	\
 	x(0,				journal_reclaim_would_deadlock)		\
-	x(EINVAL,			fsck)					\
+	x(EROFS,			fsck)					\
 	x(BCH_ERR_fsck,			fsck_ask)				\
 	x(BCH_ERR_fsck,			fsck_fix)				\
 	x(BCH_ERR_fsck,			fsck_delete_bkey)			\
@@ -186,19 +219,19 @@
 	x(BCH_ERR_recovery_will_run,	restart_recovery)			\
 	x(BCH_ERR_recovery_will_run,	cannot_rewind_recovery)			\
 	x(BCH_ERR_recovery_will_run,	recovery_pass_will_run)			\
-	x(0,				data_update_done)			\
 	x(0,				bkey_was_deleted)			\
 	x(0,				bucket_not_moveable)			\
 	x(BCH_ERR_bucket_not_moveable,	bucket_not_moveable_dev_not_rw)		\
 	x(BCH_ERR_bucket_not_moveable,	bucket_not_moveable_bucket_open)	\
 	x(BCH_ERR_bucket_not_moveable,	bucket_not_moveable_bp_mismatch)	\
 	x(BCH_ERR_bucket_not_moveable,	bucket_not_moveable_lru_race)		\
-	x(BCH_ERR_data_update_done,	data_update_done_would_block)		\
+	x(0,				data_update_done)			\
 	x(BCH_ERR_data_update_done,	data_update_done_unwritten)		\
 	x(BCH_ERR_data_update_done,	data_update_done_no_writes_needed)	\
-	x(BCH_ERR_data_update_done,	data_update_done_no_snapshot)		\
-	x(BCH_ERR_data_update_done,	data_update_done_no_dev_refs)		\
-	x(BCH_ERR_data_update_done,	data_update_done_no_rw_devs)		\
+	x(0,				data_update_fail)			\
+	x(BCH_ERR_data_update_fail,	data_update_fail_would_block)		\
+	x(BCH_ERR_data_update_fail,	data_update_fail_no_snapshot)		\
+	x(BCH_ERR_data_update_fail,	data_update_fail_no_rw_devs)		\
 	x(EINVAL,			device_state_not_allowed)		\
 	x(EINVAL,			member_info_missing)			\
 	x(EINVAL,			mismatched_block_size)			\
@@ -212,6 +245,8 @@
 	x(EINVAL,			device_already_online)			\
 	x(EINVAL,			filesystem_uuid_already_open)		\
 	x(EINVAL,			insufficient_devices_to_start)		\
+	x(EINVAL,			chardev_init_error)			\
+	x(EINVAL,			sysfs_init_error)			\
 	x(EINVAL,			invalid)				\
 	x(EINVAL,			internal_fsck_err)			\
 	x(EINVAL,			opt_parse_error)			\
@@ -225,13 +260,17 @@
 	x(EINVAL,			erasure_coding_found_btree_node)	\
 	x(EINVAL,			option_negative)			\
 	x(EINVAL,			topology_repair)			\
+	x(EINVAL,			unaligned_io)				\
 	x(BCH_ERR_topology_repair,	topology_repair_drop_this_node)		\
 	x(BCH_ERR_topology_repair,	topology_repair_drop_prev_node)		\
 	x(BCH_ERR_topology_repair,	topology_repair_did_fill_from_scan)	\
+	x(EMLINK,			too_many_links)				\
 	x(EOPNOTSUPP,			may_not_use_incompat_feature)		\
 	x(EOPNOTSUPP,			no_casefolding_without_utf8)		\
 	x(EOPNOTSUPP,			casefolding_disabled)			\
 	x(EOPNOTSUPP,			casefold_opt_is_dir_only)		\
+	x(EOPNOTSUPP,			casefolding_in_use)			\
+	x(EOPNOTSUPP,			casefold_dir_but_disabled)		\
 	x(EOPNOTSUPP,			unsupported_fsx_flag)			\
 	x(EOPNOTSUPP,			unsupported_fa_flag)			\
 	x(EOPNOTSUPP,			unsupported_fallocate_mode)		\
@@ -245,6 +284,8 @@
 	x(EROFS,			erofs_no_alloc_info)			\
 	x(EROFS,			erofs_filesystem_full)			\
 	x(EROFS,			insufficient_devices)			\
+	x(EROFS,			erofs_recovery_cancelled)		\
+	x(ESHUTDOWN,			btree_not_started)			\
 	x(0,				operation_blocked)			\
 	x(BCH_ERR_operation_blocked,	btree_cache_cannibalize_lock_blocked)	\
 	x(BCH_ERR_operation_blocked,	journal_res_blocked)			\
@@ -291,12 +332,14 @@
 	x(BCH_ERR_invalid_sb,		invalid_sb_opt_compression)		\
 	x(BCH_ERR_invalid_sb,		invalid_sb_ext)				\
 	x(BCH_ERR_invalid_sb,		invalid_sb_downgrade)			\
+	x(BCH_ERR_invalid_sb,		invalid_sb_extent_type_u64s)		\
 	x(BCH_ERR_invalid,		invalid_bkey)				\
 	x(BCH_ERR_operation_blocked,    nocow_lock_blocked)			\
 	x(EROFS,			journal_shutdown)			\
 	x(EIO,				journal_flush_err)			\
 	x(EIO,				journal_write_err)			\
 	x(EIO,				btree_node_read_err)			\
+	x(EIO,				btree_node_validate_err)		\
 	x(BCH_ERR_btree_node_read_err,	btree_node_read_err_cached)		\
 	x(EIO,				sb_not_downgraded)			\
 	x(EIO,				btree_node_write_all_failed)		\
@@ -343,31 +386,36 @@
 	x(BCH_ERR_data_read_retry_avoid,data_read_retry_ec_reconstruct_err)	\
 	x(BCH_ERR_data_read_retry_avoid,data_read_retry_csum_err)		\
 	x(BCH_ERR_data_read_retry,	data_read_retry_csum_err_maybe_userspace)\
-	x(BCH_ERR_data_read,		data_read_decompress_err)		\
-	x(BCH_ERR_data_read,		data_read_decrypt_err)			\
+	x(BCH_ERR_data_read_retry_avoid,data_read_decompress_err)		\
+	x(BCH_ERR_data_read_retry_avoid,data_read_decrypt_err)			\
 	x(BCH_ERR_data_read,		data_read_ptr_stale_race)		\
 	x(BCH_ERR_data_read_retry,	data_read_ptr_stale_retry)		\
+	x(BCH_ERR_data_read_retry,	data_read_ptr_stale_dirty)		\
 	x(BCH_ERR_data_read,		data_read_no_encryption_key)		\
 	x(BCH_ERR_data_read,		data_read_buffer_too_small)		\
 	x(BCH_ERR_data_read,		data_read_key_overwritten)		\
 	x(0,				rbio_narrow_crcs_fail)			\
-	x(BCH_ERR_btree_node_read_err,	btree_node_read_err_fixable)		\
-	x(BCH_ERR_btree_node_read_err,	btree_node_read_err_want_retry)		\
-	x(BCH_ERR_btree_node_read_err,	btree_node_read_err_must_retry)		\
-	x(BCH_ERR_btree_node_read_err,	btree_node_read_err_bad_node)		\
-	x(BCH_ERR_btree_node_read_err,	btree_node_read_err_incompatible)	\
 	x(0,				nopromote)				\
 	x(BCH_ERR_nopromote,		nopromote_may_not)			\
 	x(BCH_ERR_nopromote,		nopromote_no_rewrites)			\
 	x(BCH_ERR_nopromote,		nopromote_already_promoted)		\
 	x(BCH_ERR_nopromote,		nopromote_unwritten)			\
 	x(BCH_ERR_nopromote,		nopromote_congested)			\
+	x(BCH_ERR_nopromote,		nopromote_ratelimited)			\
 	x(BCH_ERR_nopromote,		nopromote_in_flight)			\
 	x(BCH_ERR_nopromote,		nopromote_no_writes)			\
 	x(BCH_ERR_nopromote,		nopromote_enomem)			\
 	x(0,				invalid_snapshot_node)			\
 	x(0,				option_needs_open_fs)			\
-	x(0,				remove_disk_accounting_entry)
+	x(0,				remove_disk_accounting_entry)		\
+	x(0,				nocow_trylock_fail)			\
+	x(BCH_ERR_nocow_trylock_fail,	nocow_trylock_contended)		\
+	x(BCH_ERR_nocow_trylock_fail,	nocow_trylock_bucket_full)		\
+	x(EINTR,			recovery_cancelled)			\
+	x(0,				shutdown_with_errors)			\
+	x(BCH_ERR_shutdown_with_errors,	shutdown_with_errors_fixed)		\
+	x(BCH_ERR_shutdown_with_errors,	shutdown_with_errors_unfixed)		\
+	x(BCH_ERR_shutdown_with_errors,	shutdown_with_emergency_ro)
 
 enum bch_errcode {
 	BCH_ERR_START		= 2048,
@@ -400,9 +448,8 @@ static inline long bch2_err_class(long err)
 	return err < 0 ? __bch2_err_class(err) : err;
 }
 
-#define BLK_STS_REMOVED		((__force blk_status_t)128)
-
 #include <linux/blk_types.h>
 const char *bch2_blk_status_to_str(blk_status_t);
+enum bch_errcode blk_status_to_bch_err(blk_status_t);
 
 #endif /* _BCACHFES_ERRCODE_H */
